@@ -18,6 +18,7 @@
  */
 package org.apache.axis2.jaxws.sample;
 
+import java.net.ConnectException;
 import java.net.UnknownHostException;
 import java.util.Map;
 import java.util.concurrent.ExecutionException;
@@ -26,6 +27,8 @@ import java.util.concurrent.Future;
 import javax.xml.ws.BindingProvider;
 import javax.xml.ws.Response;
 import javax.xml.ws.WebServiceException;
+import javax.xml.ws.WebServiceFeature;
+import javax.xml.ws.soap.AddressingFeature;
 import javax.xml.ws.soap.SOAPFaultException;
 
 import junit.framework.Test;
@@ -41,21 +44,31 @@ import org.apache.axis2.jaxws.sample.asyncdoclit.client.AsyncService;
 import org.apache.axis2.jaxws.sample.asyncdoclit.client.ThrowExceptionFault;
 import org.apache.axis2.jaxws.sample.asyncdoclit.common.CallbackHandler;
 import org.apache.axis2.metadata.registry.MetadataFactoryRegistry;
+import org.apache.axis2.testutils.AllTestsWithRuntimeIgnore;
+import org.junit.runner.RunWith;
 import org.test.asyncdoclit.ExceptionTypeEnum;
 import org.test.asyncdoclit.ThrowExceptionResponse;
 
 /**
  * Test for varios async exceptions whern AsyncMEP is enabled
  */
+@RunWith(AllTestsWithRuntimeIgnore.class)
 public class RuntimeExceptionsAsyncMepTest extends AbstractTestCase {
 
     private static final String DOCLITWR_ASYNC_ENDPOINT = "http://localhost:6060/axis2/services/AsyncService2.DocLitWrappedPortImplPort";
-
+    private static final String CONNECT_EXCEPTION_ENDPOINT = "http://localhost:6061/axis2/services/AsyncService2.DocLitWrappedPortImplPort";
     static final String CONNECT_404_ENDPOINT = DOCLITWR_ASYNC_ENDPOINT // Constants.DOCLITWR_ASYNC_ENDPOINT
             + "/DoesNotExist";
 
     static final String HOST_NOT_FOUND_ENDPOINT = "http://this.endpoint.does.not.exist/nope";
 
+    /*
+     * For async-on-the-wire exchanges, we need to enable WS-Addressing and get a transport
+     * listener setup to receive the inbound request from the service-provider which contains the
+     * response.  We only need to do that one time for all the tests.
+     */
+    static boolean listenerAlreadySetup = false;
+
     public static Test suite() {
         Test test = getTestSetup(new TestSuite(
                 RuntimeExceptionsAsyncMepTest.class), null,
@@ -64,9 +77,13 @@ public class RuntimeExceptionsAsyncMepTest extends AbstractTestCase {
     }
 
     private AsyncPort getPort() {
+        return getPort(null);
+    }
+    
+    private AsyncPort getPort(WebServiceFeature... features) {
 
         AsyncService service = new AsyncService();
-        AsyncPort port = service.getAsyncPort();
+        AsyncPort port = service.getAsyncPort(features);
         assertNotNull("Port is null", port);
 
         Map<String, Object> rc = ((BindingProvider) port).getRequestContext();
@@ -75,6 +92,33 @@ public class RuntimeExceptionsAsyncMepTest extends AbstractTestCase {
 
         return port;
     }
+    /**
+     * @testStrategy Invoke the proxy with sync method, specifying that it should use
+     *               the async-on-the-wire MEP.  The proxy enpdoint specifies a port that
+     *               does not exist.  Verify that the connection exception is received
+     *               by the client.
+     */
+  
+    public void testAsyncCallback_asyncWire_ConnectException() throws Exception {
+        setupAddressingAndListener();
+        
+        AsyncPort port = getPort();
+        Map<String, Object> rc = ((BindingProvider) port).getRequestContext();
+        rc.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, CONNECT_EXCEPTION_ENDPOINT);
+        rc.put("org.apache.axis2.jaxws.use.async.mep", Boolean.TRUE);
+
+        try {
+            String resp = port.throwException(ExceptionTypeEnum.WSE);
+            fail("Did not get an exception as expected");
+        } catch (Exception ee) {
+            assertTrue("ExecutionException.getCause should be an instance of WebServiceException",
+                       ee instanceof WebServiceException);
+            assertTrue("Didn't get a cause within the WebServiceException",
+                       ee.getCause() != null);
+            assertTrue("Cause was not ConnectionException as expected", 
+                       ee.getCause() instanceof ConnectException);
+        }
+    }
 
     /**
      * @testStrategy Invoke the proxy with async-polling method, the proxy is
@@ -83,6 +127,7 @@ public class RuntimeExceptionsAsyncMepTest extends AbstractTestCase {
      *               EE/WSE/UnknownHostException
      */
     public void testAsyncPolling_asyncMEP_UnknwonHost() throws Exception {
+        checkUnknownHostURL(HOST_NOT_FOUND_ENDPOINT);
 
         AsyncPort port = getPort();
 
@@ -205,6 +250,7 @@ public class RuntimeExceptionsAsyncMepTest extends AbstractTestCase {
      *               EE/WSE/UnknownHostException
      */
     public void testAsyncCallback_asyncMEP_UnknownHost() throws Exception {
+        checkUnknownHostURL(HOST_NOT_FOUND_ENDPOINT);
 
         AsyncPort port = getPort();
         Map<String, Object> rc = ((BindingProvider) port).getRequestContext();
@@ -317,20 +363,8 @@ public class RuntimeExceptionsAsyncMepTest extends AbstractTestCase {
      */
     public void testAsyncCallback_asyncMEP_asyncWire_Addressing_WebServiceException()
             throws Exception {
-
-        // we want to use addressing on the client side
-        String repopath = System.getProperty("basedir", ".") + "/"
-                + System.getProperty("build.repository");
-        String axis2xmlpath = System.getProperty("basedir", ".")
-                + "/test-resources/axis2_addressing.xml";
-        FileSystemConfigurator configurator = new FileSystemConfigurator(
-                repopath, axis2xmlpath);
-        ClientConfigurationFactory factory = new ClientConfigurationFactory(
-                configurator);
-        MetadataFactoryRegistry.setFactory(ClientConfigurationFactory.class,
-                factory);
-
-        AsyncPort port = getPort();
+        setupAddressingAndListener();
+        AsyncPort port = getPort(new AddressingFeature());
 
         Map<String, Object> rc = ((BindingProvider) port).getRequestContext();
         rc.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY,
@@ -395,5 +429,29 @@ public class RuntimeExceptionsAsyncMepTest extends AbstractTestCase {
 
         return found;
     }
+    
+    
+    /**
+     * Setup to use addressing and to start a listener to receive inbound async responses
+     * from the service-provider.
+     * 
+     * @throws Exception
+     */
+    synchronized private void setupAddressingAndListener() throws Exception {
+        if (!listenerAlreadySetup) {
+            listenerAlreadySetup = true;
+            // we want to use addressing on the client side
+            String repopath = System.getProperty("basedir", ".") + "/"
+                    + System.getProperty("build.repository", "target/client-repo");
+            String axis2xmlpath = System.getProperty("basedir", ".")
+                    + "/test-resources/axis2_addressing.xml";
+            FileSystemConfigurator configurator = new FileSystemConfigurator(
+                    repopath, axis2xmlpath);
+            ClientConfigurationFactory factory = new ClientConfigurationFactory(
+                    configurator);
+            MetadataFactoryRegistry.setFactory(ClientConfigurationFactory.class,
+                    factory);
+        }
+    }
 
 }
